==> lcmaps-1.4.28.tar.gz: lcmaps-1.4.28/src/grid_credential_handling/x509_handling/lcmaps_x509_utils.c <==
/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Gerben Venekamp <venekamp@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


/*!
    \file   lcmaps_x509_utils.c
    \brief  This file contains utility functions needed for x509 credential
            handling (openssl)
    \author Martijn Steenbakkers for EGEE
    This file contains utility functions needed for x509 credential
    handling (openssl).
    Contains code contributed by Andrew McNab.

*/

#ifndef LCMAPS_X509_UTILS_C
#define LCMAPS_X509_UTILS_C

/******************************************************************************
                             Include header files
******************************************************************************/

#define DEBUG 0
#define HACK 0

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* For X509 and STACK_OF(X509) structs (output) */
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/x509_vfy.h>
#include <openssl/pem.h>

/* globus */
#include "globus_gsi_cert_utils.h"

/* LCMAPS includes */
#include "_lcmaps_x509_utils.h"
#include "_lcmaps_log.h"

/******************************************************************************
                             Defines
******************************************************************************/



/******************************************************************************
Function:       lcmaps_x509_free_chain()
Description:    Properly free a previously allocated STACK_OF(X509) *chain
		NOTE: we need a **chain, otherwise NULL is not returned...
Parameters:
                chain: pointer to STACK_OF(X509)
******************************************************************************/
void lcmaps_x509_free_chain(
        STACK_OF(X509) **  chain
)
{
     if ( *chain )
     {
         sk_X509_pop_free( *chain, X509_free );
         /* sk_X509_free( chain ); */
         *chain = NULL;
     }
}

/******************************************************************************
Function:       lcmaps_x509_to_dn()
Description:    Get the name of the X509 cert
Parameters:
                px509: x509 certificate
Returns:        dn (which should be freed)
******************************************************************************/
char * lcmaps_x509_to_dn(
        X509 *  px509
)
{
    char * logstr = "lcmaps_x509_to_dn()";
    if (!px509)
    {
	lcmaps_log(1, "%s: No certificate found as input.\n", logstr);
	return NULL;
    }
     
    return X509_NAME_oneline(X509_get_subject_name(px509),NULL,0);
}
/******************************************************************************
Function:       lcmaps_x509_chain_to_dn()
Description:    Get the base DN of the X509 proxy cert
                Watch out! This function changes the subject DN in the X509 structure!
Parameters:
                px509: x509 certificate
                certstack: certificate chain
Returns:        dn (which should be freed)
******************************************************************************/
char * lcmaps_x509_chain_to_dn(
        X509 *  px509, STACK_OF(X509) *certstack
)
{
/*
     char * dn = NULL;
     X509_NAME * name = NULL;
     globus_result_t rc = (globus_result_t) -1;

#if 1
     if (globus_module_activate(GLOBUS_GSI_CERT_UTILS_MODULE) != GLOBUS_SUCCESS)
        return NULL;
     name = X509_get_subject_name(px509);
     if ((rc = globus_gsi_cert_utils_get_base_name(name,certstack)) != GLOBUS_SUCCESS)
     {
        fprintf(stderr, "error in globus_gsi_cert_utils_get_base_name(): rc = %d\n", (int) rc);
        return NULL;
     }
     dn = X509_NAME_oneline(name,NULL,0);
#if DEBUG
     fprintf(stderr, "lcmaps_x509_chain_to_dn(): dn = %s\n", dn);
#endif

     globus_module_deactivate(GLOBUS_GSI_CERT_UTILS_MODULE);
#else
     dn = strdup("/O=dutchgrid/O=users/O=nikhef/CN=Martijn Steenbakkers");
#if DEBUG
     fprintf(stderr, "HARDCODED DN!!!!!!: dn = %s\n", dn);
#endif
#endif
     return dn;
*/

    char *  dn = NULL;
    STACK_OF (X509) * chain = (STACK_OF(X509) *) sk_X509_dup(certstack);

    if (chain)
    {
        int amount_of_CAs = 0;
        int i = 0;
        int depth = sk_X509_num(chain);
        X509 * cert = NULL;

        /* How many CA certs are there in the chain? */
        for (i = 0; i < depth; i++)
        {
            if (cgul_x509IsCA(sk_X509_value(chain, i)))
                amount_of_CAs++;
        }

        /* Getting the user cert (filtering the CAs and such) */
        i = depth - (amount_of_CAs + 1);

	/* Next check also catches depth==0 */
	if (i < 0)
	{
	    lcmaps_log (1, "Empty certificate stack or missing user certificate, can't convert to DN\n");
	    i=0;
	    return NULL;
	}

        cert = sk_X509_value(chain, i);

        if (cert)
        {
            dn = X509_NAME_oneline(X509_get_subject_name (cert), NULL, 0); 
            lcmaps_log_debug (5, "Extracted DN: %s\n", dn);
        }
        else
        {
            lcmaps_log (1, "Couldn't extract the requested user cert from the stack, thus can't convert to DN\n");
        }

        /* clean up */
	/* sk_X509_dup DOES NOT COMPLETELY DUPLICATE THE STRUCT !!!
	 * in particular the sk->data[] arrays are not duplicated, so after the
	 * dup, certstack->data[0] == chain->data[0], not just the contents, the
	 * whole pointer. That means we should not sk_X509_pop_free chain, which
	 * also will free the ->data[] blocks, but only sk_X509_free it,
	 * otherwise the certstack->data[] blocks are freed too! See stack.c
	 * from openssl for more info */
/*	lcmaps_x509_free_chain( &chain );*/
        sk_X509_free( chain );
    }
    else
    {
        lcmaps_log (1, "Empty certificate stack, can't convert to DN\n");
    }

    return dn;
}

/******************************************************************************
Function:       lcmaps_pem_string_to_x509
Description:    Reads a X509 certificate from a PEM-encoded string
Parameters:
                px: pointer to a X509 certificate
                cert_string: PEM-encoded string 
Returns:        0 on success, non-zero otherwise
******************************************************************************/
int lcmaps_pem_string_to_x509(X509 **px, char *certstring)
{

#if 0
    BIO *certbio = NULL;

    *px = X509_new();
    if (*px == NULL) return -1;
    /* clean it first */
    X509_free(*px);

    certbio = BIO_new_mem_buf(certstring, -1);
    if (certbio)
    {
        /* This call allocates new X509 struct, so no X509_new() needed */
        *px = PEM_read_bio_X509(certbio, NULL, 0, NULL);
    }

    BIO_free(certbio);
    return 0;
#endif

    /* int rval = 0; */
    STACK_OF (X509) * chain = NULL;
    char *dn;


    if (lcmaps_pem_string_to_x509_chain(&chain, certstring) == 0)
    {
        int amount_of_CAs = 0;
        int depth = sk_X509_num(chain);
        int i = 0;
        X509 * cert = NULL;


        // How many CA certs are there in the chain?
        for (i = 0; i < depth; i++)
        {
            if (cgul_x509IsCA(sk_X509_value(chain, i)))
                amount_of_CAs++;
        }

        /* Getting the final proxy (the Leaf proxy) */
        cert = sk_X509_value(chain, 0);
        *px  = X509_dup (cert);
    }

    lcmaps_x509_free_chain( &chain ); /* note chain->NULL */

    if (!px) return -1; /* NOTE lcmaps_x509_to_dn handles *px==NULL */
    dn = lcmaps_x509_to_dn( *px );
    if (dn)
    {
        lcmaps_log_debug (5, "Got individual certificate with subject: %s\n", dn);
        free( dn );
    } else
	return -1;

    return 0;
}

/******************************************************************************
Function:       lcmaps_pem_string_to_x509_chain
Description:    Creates a dynamically allocated stack of X509 certificate objects 
                by walking through the PEM-encoded X509 certificates. 
                Copied from gridsite (Andrew McNab), original name:
                    GRSTx509StringToChain()
Parameters:
                certstack: pointer to a stack of X509 certificates
                cert_string: PEM-encoded string 
Returns:        0 on success, non-zero otherwise
******************************************************************************/
int lcmaps_pem_string_to_x509_chain(STACK_OF(X509) **certstack, char *certstring)
{
    STACK_OF(X509_INFO) *sk=NULL;
    BIO *certbio = NULL;
    X509_INFO *xi;
#if DEBUG
    int stack_size = 0;
#endif

    *certstack = sk_X509_new_null();
    if (*certstack == NULL) return -1;

    certbio = BIO_new_mem_buf(certstring, -1);
  
    if (!(sk=PEM_X509_INFO_read_bio(certbio, NULL, NULL, NULL)))
    {
        BIO_free(certbio);
        sk_X509_INFO_free(sk);
        lcmaps_x509_free_chain( certstack ); /* *certstack ->NULL*/
        return 1;
    }
  
#if DEBUG
    while (sk_X509_INFO_num(sk))
    {
        char * dn = NULL;
        fprintf(stderr, "current X509_INFO stack size: %d, DN = ", sk_X509_INFO_num(sk));
        xi=sk_X509_INFO_shift(sk);
        if (xi->x509 != NULL)
        {
            sk_X509_push(*certstack, xi->x509);
            dn = lcmaps_x509_to_dn(xi->x509);
            if (dn)
            {
                fprintf(stderr, "%s\n", dn);
                free(dn);
            }
            else
            {
                fprintf(stderr, "NULL!\n");
            }
            xi->x509=NULL;
        }
        X509_INFO_free(xi);
    }

    stack_size = sk_X509_INFO_num(sk);
    fprintf(stderr, "final size of stack of X509_INFO: %d, so it's %s\n",
        stack_size, (stack_size?"not yet empty! (NOT OK)":"empty (OK)."));
    fprintf(stderr, "size of stack of X509: %d\n", sk_X509_num(*certstack));
    {
        char * subject = NULL;
        X509 * cert = NULL;
        int i = 0;

        for (i = 0; i < sk_X509_num(*certstack); i++)
        {
            fprintf(stderr, "    dn of cert %d: ", i);
            cert = sk_X509_value(*certstack, i);
            subject = lcmaps_x509_to_dn(cert);
            if (subject)
            {
                fprintf(stderr, "%s\n", subject);
                free(subject);
            }
            else
            {
                fprintf(stderr, "NULL!\n");
            }
        }
    }
#else
    while (sk_X509_INFO_num(sk))
    {
        xi=sk_X509_INFO_shift(sk);
        if (xi->x509 != NULL)
        {
            sk_X509_push(*certstack, xi->x509);
            xi->x509=NULL;
        }
        X509_INFO_free(xi);
    }
#endif


   if (!sk_X509_num(*certstack))
     {
       BIO_free(certbio);
       sk_X509_INFO_free(sk);
       lcmaps_x509_free_chain( certstack ); /* *certstack -> NULL */
       return 1;
     }

   BIO_free(certbio);
   sk_X509_INFO_free(sk);
   
   return 0;
}


/******************************************************************************
Function:   lcmaps_x509IsCA
Description:
    Tests if the X509 * cert is a CA certificate or not
Parameters:
    A X509 pointer
Returns:
    0      : Not a CA cert
    1      : This is a CA cert
******************************************************************************/
int lcmaps_x509IsCA (X509 *cert)
{
    return cgul_x509IsCA (cert);
}


/******************************************************************************
Function:   cgul_x509IsCA
Description:
    Tests if the X509 * cert is a CA certificate or not
Parameters:
    A X509 pointer
Returns:
    0      : Not a CA cert
    1      : This is a CA cert
******************************************************************************/
int cgul_x509IsCA (X509 *cert)
{
   int purpose_id;

   purpose_id = X509_PURPOSE_get_by_sname("sslclient");

   /* final argument to X509_check_purpose() is whether to check for CAness */

   if (X509_check_purpose(cert, purpose_id + X509_PURPOSE_MIN, 1))
        return 1;
   else return 0;
}


/******************************************************************************
Function:       cgul_x509_select_eec_from_chain()
Description:    Get the End-Entity Certificate from a certificate chain
Parameters:
                certstack: certificate chain
Returns:        pointer to X509 EEC
******************************************************************************/
X509 * cgul_x509_select_eec_from_chain (STACK_OF(X509) * chain)
{
    int amount_of_CAs = 0;
    int i             = 0;
    int depth         = 0;

    /* I must have a certificate chain to work with */
    if (!chain)
    {
        return NULL;
    }
    else
    {
        /* How many elements are there to the chain? */
        depth = sk_X509_num(chain);

        /* How many CA certs are there in the chain? */
        for (i = 0; i < depth; i++)
        {
            if (cgul_x509IsCA(sk_X509_value(chain, i)))
                amount_of_CAs++;
        }

        /* Getting the user cert (filtering the CAs and such) */
        i = depth - (amount_of_CAs + 1);

        /* Next check also catches depth==0 */
        if (i < 0)
        {
            /* Counting the stack went wrong. Errornous stack? */
            return NULL;
        }

        /* The End-Entity Certificate */
        return sk_X509_value(chain, i);
    }
}


/******************************************************************************
Function:       cgul_x509_select_final_cert_from_chain()
Description:    Get the Final Delegation from a certificate chain
Parameters:
                certstack: certificate chain
Returns:        pointer to X509 final delegation
******************************************************************************/
X509 * cgul_x509_select_final_cert_from_chain (STACK_OF(X509) * chain)
{
    int amount_of_CAs = 0;
    int i             = 0;
    int depth         = 0;

    /* I must have a certificate chain to work with */
    if (!chain)
    {
        return NULL;
    }
    else
    {
        /* How many elements are there to the chain? */
        depth = sk_X509_num(chain);

        /* How many CA certs are there in the chain? */
        for (i = 0; i < depth; i++)
        {
            if (cgul_x509IsCA(sk_X509_value(chain, i)))
                amount_of_CAs++;
        }

        /* Must have at least an EEC, possibly a delegation */
        if (depth > amount_of_CAs)
        {
            /* Selecting EEC or final delegation */
            return sk_X509_value(chain, 0);
        }
        else
        {
            /* Only CA files found */
            return NULL;
        }
    }
}

/******************************************************************************
Function:       cgul_x509_chain_to_subject_dn()
Description:    Get the Subject DN of the End-Entity Certificate of the X509 cert
Parameters:
                certstack: certificate chain
Returns:        Subject DN string (which should be freed)
******************************************************************************/
char * cgul_x509_chain_to_subject_dn(STACK_OF(X509) * chain)
{
    X509 * eec_cert = NULL;

    if ((eec_cert = cgul_x509_select_eec_from_chain (chain)) == NULL)
    {
        return NULL;
    }
    else
    {
        /* You must FREE the DN after use */
        return X509_NAME_oneline(X509_get_subject_name (eec_cert), NULL, 0);
    }
}

/******************************************************************************
Function:       cgul_x509_chain_to_issuer_dn()
Description:    Get the Issuer DN of the End-Entity Certificate of the X509 cert
Parameters:
                certstack: certificate chain
Returns:        Issuer DN string (which should be freed)
******************************************************************************/
char * cgul_x509_chain_to_issuer_dn(STACK_OF(X509) * chain)
{
    X509 * eec_cert = NULL;

    if ((eec_cert = cgul_x509_select_eec_from_chain (chain)) == NULL)
    {
        return NULL;
    }
    else
    {
        /* You must FREE the Issuer DN after use */
        return X509_NAME_oneline(X509_get_issuer_name (eec_cert), NULL, 0);
    }
}
#endif /* LCMAPS_X509_UTILS_C */

/******************************************************************************
CVS Information:
    $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/x509_handling/lcmaps_x509_utils.c,v $
    $Date: 2010-12-23 14:35:40 +0100 (do, 23 dec 2010) $
    $Revision: 11815 $
    $Author: okoeroo $
******************************************************************************/

==> lcmaps-1.6.4.tar.gz: lcmaps-1.6.4/src/grid_credential_handling/x509_handling/lcmaps_x509_utils.c <==
/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl>
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Gerben Venekamp <venekamp@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


/*!
    \file   lcmaps_x509_utils.c
    \brief  This file contains utility functions needed for x509 credential
            handling (openssl)
    \author Martijn Steenbakkers for EGEE
    This file contains utility functions needed for x509 credential
    handling (openssl).
    Contains code contributed by Andrew McNab.

*/

#ifndef LCMAPS_X509_UTILS_C
#define LCMAPS_X509_UTILS_C

/******************************************************************************
                             Include header files
******************************************************************************/

#if 0
#define DEBUG 0
#define HACK 0
#endif

/* Needed for NULL */
#include <stdio.h>

/* For X509 and STACK_OF(X509) structs (output) */
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/x509_vfy.h>
#include <openssl/pem.h>
#include <openssl/bio.h>

/* LCMAPS includes */
#include "_lcmaps_x509_utils.h"
#include "lcmaps_log.h"

/******************************************************************************
                             Defines
******************************************************************************/



/******************************************************************************
Function:       lcmaps_x509_free_chain()
Description:    Properly free a previously allocated STACK_OF(X509) *chain
                NOTE: we need a **chain, otherwise NULL is not returned...
Parameters:
                chain: pointer to STACK_OF(X509)
******************************************************************************/
void lcmaps_x509_free_chain(
        STACK_OF(X509) **  chain
)
{
     if (chain && *chain) {
         sk_X509_pop_free(*chain, X509_free);
         *chain = NULL;
     }
}

/******************************************************************************
Function:       lcmaps_x509_to_dn()
Description:    Get the name of the X509 cert
Parameters:
                px509: x509 certificate
Returns:        dn (which should be freed)
******************************************************************************/
char * lcmaps_x509_to_dn(
        X509 *  px509
)
{
    if (!px509)
    {
        lcmaps_log(LOG_ERR, "%s(): No certificate found as input.\n", __func__);
        return NULL;
    }
    return X509_NAME_oneline(X509_get_subject_name(px509),NULL,0);
}
/******************************************************************************
Function:       lcmaps_x509_chain_to_dn()
Description:    Get the base DN of the X509 proxy cert
                Watch out! This function changes the subject DN in the X509 structure!
Parameters:
                px509: x509 certificate
                certstack: certificate chain
Returns:        dn (which should be freed)
******************************************************************************/
char * lcmaps_x509_chain_to_dn(
        X509 *  px509, STACK_OF(X509) *certstack
)
{
    return cgul_x509_chain_to_subject_dn(certstack);
}

/******************************************************************************
Function:       lcmaps_pem_string_to_x509
Description:    Reads a X509 certificate from a PEM-encoded string and returns
                the first X509 in the stack, which is the last certificate in
                the chain (EEC or final/leaf certificate/delegation)
Parameters:
                px: pointer to an allocated X509 certificate. (needs free)
                cert_string: PEM-encoded string
Returns:        0 on success, non-zero otherwise
******************************************************************************/
int lcmaps_pem_string_to_x509(X509 **px, char *certstring)
{
    /* int rval = 0; */
    STACK_OF (X509) * chain = NULL;
    X509 *cert = NULL;

    if (!px) {
        lcmaps_log(LOG_ERR, "%s: Error: No output pointer provided.\n", __func__);
        return -1;
    }

    if (lcmaps_pem_string_to_x509_chain(&chain, certstring) == 0) {
        /* Getting the final proxy (the Leaf proxy) */
        cert = sk_X509_value(chain, 0);
        *px = X509_dup(cert);
        if (!(*px)) {
            lcmaps_x509_free_chain(&chain); /* note chain->NULL */
            return -1;
        }
        lcmaps_x509_free_chain(&chain); /* note chain->NULL */
    } else {
        return -1;
    }
    return 0;
}

/******************************************************************************
Function:       lcmaps_pem_string_to_x509_chain
Description:    Creates a dynamically allocated stack of X509 certificate objects
                by walking through the PEM-encoded X509 certificates.
                Copied from gridsite (Andrew McNab), original name:
                    GRSTx509StringToChain()
Parameters:
                certstack: pointer to a stack of X509 certificates
                cert_string: PEM-encoded string
Returns:        0 on success, non-zero otherwise
******************************************************************************/
int lcmaps_pem_string_to_x509_chain(STACK_OF(X509) **certstack, char *certstring)
{
    STACK_OF(X509_INFO) *sk=NULL;
    BIO *certbio = NULL;
    X509_INFO *xi;

    *certstack = sk_X509_new_null();
    if (*certstack == NULL) return -1;

    certbio = BIO_new_mem_buf(certstring, -1);

    if (!(sk=PEM_X509_INFO_read_bio(certbio, NULL, NULL, NULL)))
    {
        BIO_free(certbio);
        sk_X509_INFO_free(sk);
        lcmaps_x509_free_chain( certstack ); /* *certstack ->NULL*/
        return 1;
    }

    while (sk_X509_INFO_num(sk))
    {
        xi=sk_X509_INFO_shift(sk);
        if (xi->x509 != NULL)
        {
            sk_X509_push(*certstack, xi->x509);
            xi->x509=NULL;
        }
        X509_INFO_free(xi);
    }

    if (!sk_X509_num(*certstack))
    {
        BIO_free(certbio);
        sk_X509_INFO_free(sk);
        lcmaps_x509_free_chain( certstack ); /* *certstack -> NULL */
        return 1;
    }

   BIO_free(certbio);
   sk_X509_INFO_free(sk);

   return 0;
}


/******************************************************************************
Function:   lcmaps_x509IsCA
Description:
    Tests if the X509 * cert is a CA certificate or not
Parameters:
    A X509 pointer
Returns:
    0      : Not a CA cert
    1      : This is a CA cert
******************************************************************************/
int lcmaps_x509IsCA (X509 *cert)
{
    return cgul_x509IsCA (cert);
}


/******************************************************************************
Function:   cgul_x509IsCA
Description:
    Tests if the X509 * cert is a CA certificate or not
Parameters:
    A X509 pointer
Returns:
    0      : Not a CA cert
    1      : This is a CA cert
******************************************************************************/
int cgul_x509IsCA (X509 *cert)
{
   int purpose_id;

   purpose_id = X509_PURPOSE_get_by_sname("sslclient");

   /* final argument to X509_check_purpose() is whether to check for CAness */

   if (X509_check_purpose(cert, purpose_id + X509_PURPOSE_MIN, 1))
        return 1;
   else return 0;
}


/******************************************************************************
Function:       cgul_x509_select_eec_from_chain()
Description:    Get the End-Entity Certificate from a certificate chain
Parameters:
                certstack: certificate chain
Returns:        pointer to X509 EEC
******************************************************************************/
X509 * cgul_x509_select_eec_from_chain (STACK_OF(X509) * chain)
{
    int amount_of_CAs = 0;
    int i             = 0;
    int depth         = 0;

    /* I must have a certificate chain to work with */
    if (!chain)
    {
        lcmaps_log(LOG_DEBUG, "%s: Empty chain, nothing to do\n", __func__);
        return NULL;
    }
    else
    {
        /* How many elements are there to the chain? */
        depth = sk_X509_num(chain);


        /* How many CA certs are there in the chain? */
        for (i = 0; i < depth; i++)
        {
            if (cgul_x509IsCA(sk_X509_value(chain, i)))
                amount_of_CAs++;
        }

        /* Getting the user cert (filtering the CAs and such) */
        i = depth - (amount_of_CAs + 1);

        /* Next check also catches depth==0 */
        if (i < 0)
        {
            /* Counting the stack went wrong. Errornous stack? */
            return NULL;
        }

        /* The End-Entity Certificate */
        return sk_X509_value(chain, i);
    }
}


/******************************************************************************
Function:       cgul_x509_select_final_cert_from_chain()
Description:    Get the Final Delegation from a certificate chain
Parameters:
                certstack: certificate chain
Returns:        pointer to X509 final delegation
******************************************************************************/
X509 * cgul_x509_select_final_cert_from_chain (STACK_OF(X509) * chain)
{
    int amount_of_CAs = 0;
    int i             = 0;
    int depth         = 0;

    /* I must have a certificate chain to work with */
    if (!chain)
    {
        return NULL;
    }
    else
    {
        /* How many elements are there to the chain? */
        depth = sk_X509_num(chain);

        /* How many CA certs are there in the chain? */
        for (i = 0; i < depth; i++)
        {
            if (cgul_x509IsCA(sk_X509_value(chain, i)))
                amount_of_CAs++;
        }

        /* Must have at least an EEC, possibly a delegation */
        if (depth > amount_of_CAs)
        {
            /* Selecting EEC or final delegation */
            return sk_X509_value(chain, 0);
        }
        else
        {
            /* Only CA files found */
            return NULL;
        }
    }
}

/******************************************************************************
Function:       cgul_x509_chain_to_subject_dn()
Description:    Get the Subject DN of the End-Entity Certificate of the X509 cert
Parameters:
                certstack: certificate chain
Returns:        Subject DN string (which should be freed)
******************************************************************************/
char * cgul_x509_chain_to_subject_dn(STACK_OF(X509) * chain)
{
    X509 * eec_cert = NULL;

    /* I need input */
    if (chain == NULL) {
        return NULL;
    }

    /* Select the End-Entity Certificate from the certificate chain */
    eec_cert = cgul_x509_select_eec_from_chain (chain);
    if (eec_cert == NULL) {
        lcmaps_log(LOG_DEBUG, "%s: No EEC found in the certificate chain.\n", __func__);
        return NULL;
    } else {
        /* You must FREE the DN after use */
        return X509_NAME_oneline(X509_get_subject_name(eec_cert), NULL, 0);
    }
}

/******************************************************************************
Function:       cgul_x509_chain_to_issuer_dn()
Description:    Get the Issuer DN of the End-Entity Certificate of the X509 cert
Parameters:
                certstack: certificate chain
Returns:        Issuer DN string (which should be freed)
******************************************************************************/
char * cgul_x509_chain_to_issuer_dn(STACK_OF(X509) * chain)
{
    X509 * eec_cert = NULL;

    if ((eec_cert = cgul_x509_select_eec_from_chain (chain)) == NULL)
    {
        return NULL;
    }
    else
    {
        /* You must FREE the Issuer DN after use */
        return X509_NAME_oneline(X509_get_issuer_name (eec_cert), NULL, 0);
    }
}
#endif /* LCMAPS_X509_UTILS_C */

/******************************************************************************
CVS Information:
    $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/src/grid_credential_handling/x509_handling/lcmaps_x509_utils.c,v $
    $Date: 2014-02-26 16:01:45 +0100 (Wed, 26 Feb 2014) $
    $Revision: 17529 $
    $Author: msalle $
******************************************************************************/
